<?php

namespace WHMCS\Module\Server\VirtFusionResourcePack;

class ModuleFunctions extends Module
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * CREATE USER
     *
     */
    public function createAccount($params)
    {
        try {

            /**
             *
             * If the service exists in the custom table, Cancel the create account action.
             *
             */
            if (Database::checkSystemService($params['serviceid'])) {
                return 'Service already exists. You must run a termination first.';
            }


            /**
             *
             * If no VirtFusionResourcePack control server exists, cancel the create account action.
             *
             */

            $server = $params['serverid'] ?: false;
            $cp = $this->getCP($server, $server ? false : true);

            if (!$cp) {
                return 'No Control server found.';
            }

            Log::insert(__FUNCTION__, $params, []);

            /**
             *
             * Does a user account in VirtFusion match this account (byExtRelationId) in WHMCS.
             *
             */
            $request = $this->initCurl($cp['token']);

            $data = $request->get($cp['url'] . '/users/' . $params['userid'] . '/byExtRelation');

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {
                case 200:

                    /**
                     *
                     * A user with relation ID exists in VirtFusion. We can add a pack under that account.
                     *
                     */
                    break;

                case 404:

                    /**
                     *
                     * A user doesn't exist in VirtFusion. We should attempt to create one.
                     *
                     */
                    $user = Database::getUser($params['userid']);

                    $request = $this->initCurl($cp['token']);

                    $request->addOption(CURLOPT_POSTFIELDS, json_encode(
                        [
                            "name" => $user->firstname . ' ' . $user->lastname,
                            "email" => $user->email,
                            "extRelationId" => $user->id
                        ]
                    ));

                    $data = $request->post($cp['url'] . '/users');

                    Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

                    if ($request->getRequestInfo('http_code') !== 201) {
                        return 'Unable to create user.';
                    }
                    break;
                default:
                    return 'Error processing user account.';
                    break;
            }

            $data = json_decode($data);

            $request = $this->initCurl($cp['token']);
            $request->addOption(CURLOPT_POSTFIELDS, json_encode(['syncToProfiles' => true]));
            $request->put($cp['url'] . '/selfService/access/byUserExtRelationId/' . $params['userid']);

            /**
             *
             * A user is available. We can now add a resource pack.
             *
             */

            $options = [
                "packId" => $params['configoption1'],
                "enabled" => true,
            ];


            $request = $this->initCurl($cp['token']);
            $request->addOption(CURLOPT_POSTFIELDS, json_encode($options));

            $data = $request->post($cp['url'] . '/selfService/resourcePack/byUserExtRelationId/' . $params['userid']);

            $data = json_decode($data);

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            if ($request->getRequestInfo('http_code') === 201) {

                Database::systemOnPackAdd($params['serviceid'], $data);

                /**
                 * Sync profiles to make sure the user has access to resource packs in VirtFusion.
                 */
                $request = $this->initCurl($cp['token']);
                $request->addOption(CURLOPT_POSTFIELDS, json_encode(['syncToProfiles' => true]));
                $request->put($cp['url'] . '/selfService/access/byUserExtRelationId/' . $params['userid']);

                /**
                 *
                 * Resource Pack was added successfully.
                 *
                 */
                return 'success';
            } else {
                if ($data->errors[0]) {
                    return $data->errors[0];
                }
                return 'Unknown error.';
            }
        } catch (\Exception $e) {
            Log::insert(__FUNCTION__, $params, $e->getMessage());
            return $e->getMessage();
        }
    }

    /**
     *
     * TERMINATE RESOURCE PACK
     *
     */
    public function terminateAccount($params)
    {

        $service = Database::getSystemService($params['serviceid']);

        if ($service) {

            $whmcsService = Database::getWhmcsService($params['serviceid']);

            $cp = $this->getCP($whmcsService->server);

            $request = $this->initCurl($cp['token']);

            if ($params['configoption3'] == 'Delete Servers') {
                $data = $request->delete($cp['url'] . '/selfService/resourcePackServers/' . $service->pack_id . '?delay=' . $params['configoption1']);
            } else {
                $data = $request->post($cp['url'] . '/selfService/resourcePackServers/' . $service->pack_id . '/suspend');
            }

            $data = json_decode($data);

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    break;

                case 423:
                    if (property_exists($data, 'msg')) {
                        return $data->msg;
                    }

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'pack not found') {
                            Database::deleteSystemService($params['serviceid']);
                            return 'success';
                        } else {
                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process a termination.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;

                default:
                    return 'Termination request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }

            $request = $this->initCurl($cp['token']);


            $data = $request->delete($cp['url'] . '/selfService/resourcePack/' . $service->pack_id . '?disable=true');
            $data = json_decode($data);

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                case 424:

                    Database::deleteSystemService($params['serviceid']);
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'pack not found') {
                            Database::deleteSystemService($params['serviceid']);
                            return 'success';
                        } else {
                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process a termination.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;

                default:
                    return 'Termination request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found. Termination routine has already been run?';
    }

    /**
     *
     * SUSPEND
     *
     */
    public function suspendAccount($params)
    {
        $service = Database::getSystemService($params['serviceid']);

        if ($service) {

            $whmcsService = Database::getWhmcsService($params['serviceid']);

            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->post($cp['url'] . '/selfService/resourcePackServers/' . $service->pack_id . '/suspend');
            $data = json_decode($data);

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'pack not found') {
                            Database::deleteSystemService($params['serviceid']);
                            return 'success';
                        } else {

                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process a suspension.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;
                case 423:
                    if (property_exists($data, 'msg')) {
                        return $data->msg;
                    }

                default:
                    return 'Suspend request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found.';
    }

    public function unsuspendAccount($params)
    {
        $service = Database::getSystemService($params['serviceid']);

        if ($service) {
            $whmcsService = Database::getWhmcsService($params['serviceid']);

            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->post($cp['url'] . '/selfService/resourcePackServers/' . $service->pack_id . '/unsuspend');
            $data = json_decode($data);

            Log::insert(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'pack not found') {
                            Database::deleteSystemService($params['serviceid']);
                            return 'success';
                        } else {
                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process an unsuspension.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;
                case 423:
                case 409:
                    if (property_exists($data, 'msg')) {
                        return $data->msg;
                    }

                default:
                    return 'Unsuspend request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found';
    }

    public function adminServicesTabFields($params)
    {
        $packId = '';

        $service = Database::getSystemService($params['serviceid']);
        $systemUrl = Database::getSystemUrl();

        if ($service) {
            $packId = $service->pack_id;
        }

        $fields = [
            'Pack ID' => AdminHTML::packId($systemUrl, $packId),
            '' => AdminHTML::packInfo($systemUrl, $params['serviceid']),
        ];

        if ($params['status'] != 'Terminated') {
            $fields['Options'] = AdminHTML::options($systemUrl, $params['serviceid']);
        }

        return $fields;
    }

    public function adminServicesTabFieldsSave($params)
    {

        if ($_POST['modulefields'][0] == '') {
            Database::deleteSystemService($params['serviceid']);
        } else {
            Database::updateSystemServicePackId($params['serviceid'], $_POST['modulefields'][0]);
        }
    }

    public function clientArea($params)
    {
        $serverHostname = null;
        if (array_key_exists('serverhostname', $params)) {
            $serverHostname = $params['serverhostname'];
        }

        try {
            return [
                'tabOverviewReplacementTemplate' => 'overview',
                'templateVariables' => [
                    'systemURL' => Database::getSystemUrl(),
                    'serviceStatus' => $params['status'],
                    'serverHostname' => $serverHostname,
                ],
            ];
        } catch (\Throwable $e) {

            Log::insert(__FUNCTION__, $params, $e->getMessage());

            return [
                'tabOverviewReplacementTemplate' => 'error',
                'templateVariables' => [],
            ];
        }
    }
}